package sun.security.rsa;

import java.security.NoSuchAlgorithmException;
import sun.security.util.DerOutputStream;
import sun.security.util.DerValue;
import sun.security.x509.AlgorithmId;
import java.io.IOException;
import sun.security.util.DerInputStream;
import java.security.spec.InvalidParameterSpecException;
import java.security.spec.MGF1ParameterSpec;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.PSSParameterSpec;
import java.security.AlgorithmParametersSpi;

public final class PSSParameters extends AlgorithmParametersSpi
{
    private PSSParameterSpec spec;
    
    public PSSParameters() {
    }
    
    @Override
    protected void engineInit(AlgorithmParameterSpec algorithmParameterSpec) throws InvalidParameterSpecException {
        if (!(algorithmParameterSpec instanceof PSSParameterSpec)) {
            throw new InvalidParameterSpecException("Inappropriate parameter specification");
        }
        PSSParameterSpec spec = (PSSParameterSpec)algorithmParameterSpec;
        String mgfAlgorithm = spec.getMGFAlgorithm();
        if (!spec.getMGFAlgorithm().equalsIgnoreCase("MGF1")) {
            throw new InvalidParameterSpecException("Unsupported mgf " + mgfAlgorithm + "; MGF1 only");
        }
        if (!(spec.getMGFParameters() instanceof MGF1ParameterSpec)) {
            throw new InvalidParameterSpecException("Inappropriate mgf parameters; non-null MGF1ParameterSpec only");
        }
        this.spec = spec;
        return;
    }
    
    @Override
    protected void engineInit(byte[] data) throws IOException {
        String mdName = PSSParameterSpec.DEFAULT.getDigestAlgorithm();
        MGF1ParameterSpec mgfSpec = (MGF1ParameterSpec)PSSParameterSpec.DEFAULT.getMGFParameters();
        int saltLen = PSSParameterSpec.DEFAULT.getSaltLength();
        int n = PSSParameterSpec.DEFAULT.getTrailerField();
        DerValue[] sequence;
        for (DerValue derValue : sequence = new DerInputStream(data).getSequence(4)) {
            if (derValue.isContextSpecific((byte)0)) {
                mdName = AlgorithmId.parse(derValue.data.getDerValue()).getName();
            }
            else if (derValue.isContextSpecific((byte)1)) {
                AlgorithmId parse = AlgorithmId.parse(derValue.data.getDerValue());
                if (!parse.getOID().equals(AlgorithmId.mgf1_oid)) {
                    throw new IOException("Only MGF1 mgf is supported");
                }
                String name = AlgorithmId.parse(new DerValue(parse.getEncodedParams())).getName();
                int n2 = -1;
                switch (name.hashCode()) {
                    case 78861104: {
                        if (name.equals("SHA-1")) {
                            n2 = 0;
                            break;
                        }
                        break;
                    }
                    case -1523887821: {
                        if (name.equals("SHA-224")) {
                            n2 = 1;
                            break;
                        }
                        break;
                    }
                    case -1523887726: {
                        if (name.equals("SHA-256")) {
                            n2 = 2;
                            break;
                        }
                        break;
                    }
                    case -1523886674: {
                        if (name.equals("SHA-384")) {
                            n2 = 3;
                            break;
                        }
                        break;
                    }
                    case -1523884971: {
                        if (name.equals("SHA-512")) {
                            n2 = 4;
                            break;
                        }
                        break;
                    }
                    case 752961850: {
                        if (name.equals("SHA-512/224")) {
                            n2 = 5;
                            break;
                        }
                        break;
                    }
                    case 752961945: {
                        if (name.equals("SHA-512/256")) {
                            n2 = 6;
                            break;
                        }
                        break;
                    }
                }
                switch (n2) {
                    case 0: {
                        mgfSpec = MGF1ParameterSpec.SHA1;
                        break;
                    }
                    case 1: {
                        mgfSpec = MGF1ParameterSpec.SHA224;
                        break;
                    }
                    case 2: {
                        mgfSpec = MGF1ParameterSpec.SHA256;
                        break;
                    }
                    case 3: {
                        mgfSpec = MGF1ParameterSpec.SHA384;
                        break;
                    }
                    case 4: {
                        mgfSpec = MGF1ParameterSpec.SHA512;
                        break;
                    }
                    case 5: {
                        mgfSpec = MGF1ParameterSpec.SHA512_224;
                        break;
                    }
                    case 6: {
                        mgfSpec = MGF1ParameterSpec.SHA512_256;
                        break;
                    }
                    default: {
                        throw new IOException("Unrecognized message digest algorithm " + name);
                    }
                }
            }
            else if (derValue.isContextSpecific((byte)2)) {
                saltLen = derValue.data.getDerValue().getInteger();
                if (saltLen < 0) {
                    throw new IOException("Negative value for saltLength");
                }
            }
            else if (derValue.isContextSpecific((byte)3)) {
                n = derValue.data.getDerValue().getInteger();
                if (n != 1) {
                    throw new IOException("Unsupported trailerField value " + n);
                }
            }
            else {
                throw new IOException("Invalid encoded PSSParameters");
            }
        }
        this.spec = new PSSParameterSpec(mdName, "MGF1", mgfSpec, saltLen, n);
    }
    
    @Override
    protected void engineInit(byte[] array, String s) throws IOException {
        if (s != null && !s.equalsIgnoreCase("ASN.1")) {
            throw new IllegalArgumentException("Only support ASN.1 format");
        }
        this.engineInit(array);
        return;
    }
    
    @Override
    protected <T extends AlgorithmParameterSpec> T engineGetParameterSpec(Class<T> clazz) throws InvalidParameterSpecException {
        if (PSSParameterSpec.class.isAssignableFrom(clazz)) {
            return (T)clazz.cast(this.spec);
        }
        throw new InvalidParameterSpecException("Inappropriate parameter specification");
    }
    
    @Override
    protected byte[] engineGetEncoded() throws IOException {
        return getEncoded(this.spec);
    }
    
    @Override
    protected byte[] engineGetEncoded(String s) throws IOException {
        if (s != null && !s.equalsIgnoreCase("ASN.1")) {
            throw new IllegalArgumentException("Only support ASN.1 format");
        }
        return this.engineGetEncoded();
    }
    
    @Override
    protected String engineToString() {
        return this.spec.toString();
    }
    
    public static byte[] getEncoded(PSSParameterSpec pssParameterSpec) throws IOException {
        AlgorithmParameterSpec mgfParameters = pssParameterSpec.getMGFParameters();
        if (!(mgfParameters instanceof MGF1ParameterSpec)) {
            throw new IOException("Cannot encode " + mgfParameters);
        }
        MGF1ParameterSpec mgf1ParameterSpec = (MGF1ParameterSpec)mgfParameters;
        DerOutputStream derOutputStream = new DerOutputStream();
        AlgorithmId value;
        try {
            value = AlgorithmId.get(pssParameterSpec.getDigestAlgorithm());
        }
        catch (NoSuchAlgorithmException ex) {
            throw new IOException("AlgorithmId " + pssParameterSpec.getDigestAlgorithm() + " impl not found");
        }
        if (!value.getOID().equals(AlgorithmId.SHA_oid)) {
            DerOutputStream derOutputStream2 = new DerOutputStream();
            value.derEncode(derOutputStream2);
            derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)0), derOutputStream2);
        }
        AlgorithmId value2;
        try {
            value2 = AlgorithmId.get(mgf1ParameterSpec.getDigestAlgorithm());
        }
        catch (NoSuchAlgorithmException ex2) {
            throw new IOException("AlgorithmId " + mgf1ParameterSpec.getDigestAlgorithm() + " impl not found");
        }
        if (!value2.getOID().equals(AlgorithmId.SHA_oid)) {
            DerOutputStream out = new DerOutputStream();
            out.putOID(AlgorithmId.mgf1_oid);
            value2.encode(out);
            DerOutputStream o = new DerOutputStream();
            o.write((byte)48, out);
            derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)1), o);
        }
        if (pssParameterSpec.getSaltLength() != 20) {
            DerOutputStream o2 = new DerOutputStream();
            o2.putInteger(pssParameterSpec.getSaltLength());
            derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)2), o2);
        }
        if (pssParameterSpec.getTrailerField() != 1) {
            DerOutputStream o3 = new DerOutputStream();
            o3.putInteger(pssParameterSpec.getTrailerField());
            derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)3), o3);
        }
        DerOutputStream o4 = new DerOutputStream();
        o4.write((byte)48, derOutputStream);
        return o4.toByteArray();
    }
}
